<?php
// $Id: format_currency_admin.inc,v 1.1 2010/10/22 14:57:35 pobster Exp $

/**
 * @file
 * Administration file for the Format Currencies project.
 */

/**
 * Settings form.
 */
function format_currency_settings(&$form_state) {
  $form['format_currency_display_code'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display Currency Code'),
    '#description' => t('Whether to display the Currency Code (eg. "USD $123.45") before a formatted price.'),
    '#default_value' => variable_get('format_currency_display_code', FALSE),
  );
  $form['format_currency_display_symbol'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use symbol rather than HTML entity'),
    '#description' => t('The database table holds both a symbol and HTML entity for Currencies, check the box to use the symbol over the entity.'),
    '#default_value' => variable_get('format_currency_display_symbol', FALSE),
  );

  return system_settings_form($form);
}

/**
 * Overview of currencies currently in the database.
 */
function format_currency_admin(&$form_state) {
  $form['currencies']['#tree'] = TRUE;

  $query = pager_query("SELECT * FROM {currencies}". tablesort_sql(_format_currency_admin_header()), 25);

  while ($currency = db_fetch_object($query)) {
    $form['currencies'][$currency->code]['code'] = array(
      '#type' => 'markup',
      '#value' => $currency->code,
    );
    $form['currencies'][$currency->code]['symbol'] = array(
      '#type' => 'markup',
      '#value' => $currency->symbol,
    );
    $form['currencies'][$currency->code]['entity'] = array(
      '#type' => 'markup',
      '#value' => $currency->entity,
    );
    $form['currencies'][$currency->code]['decimal_places'] = array(
      '#type' => 'markup',
      '#value' => $currency->decimal_places,
    );
    $form['currencies'][$currency->code]['decimal_separator'] = array(
      '#type' => 'markup',
      '#value' => $currency->decimal_separator,
    );
    $form['currencies'][$currency->code]['thousands_separator'] = array(
      '#type' => 'markup',
      '#value' => $currency->thousands_separator,
    );
    $form['currencies'][$currency->code]['symbol_position'] = array(
      '#type' => 'markup',
      '#value' => $currency->symbol_position,
    );
  }

  return $form;
}

/**
 * Adds a new currency to the database.
 */
function format_currency_add(&$form_state) {
  $form['code'] = array(
    '#type' => 'textfield',
    '#title' => t('Currency code'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
  );
  $form['symbol'] = array(
    '#type' => 'textfield',
    '#title' => t('Symbol'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
  );
  $form['entity'] = array(
    '#type' => 'textfield',
    '#title' => t('HTML Entity'),
    '#size' => 10,
    '#maxlength' => 10,
  );
  $form['decimal_places'] = array(
    '#type' => 'textfield',
    '#title' => t('Decimal Places'),
    '#required' => TRUE,
    '#size' => 3,
    '#maxlength' => 2,
    '#default_value' => 2,
  );
  $form['decimal_separator'] = array(
    '#type' => 'textfield',
    '#title' => t('Decimal Separator'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => '.',
  );
  $form['thousands_separator'] = array(
    '#type' => 'textfield',
    '#title' => t('Thousands Separator'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => ',',
  );
  $form['symbol_position'] = array(
    '#type' => 'select',
    '#title' => t('Symbol Position'),
    '#options' => array(
      FORMAT_CURRENCY_LEFT => t('Left'),
      FORMAT_CURRENCY_RIGHT => t('Right'),
    ),
    '#required' => TRUE,
  );

  return confirm_form($form, t('Add New Currency'), 'admin/settings/format_currency/overview', ' ', t('Add'));
}

/**
 * Validation callback for the format_currency_add form.
 */
function format_currency_add_validate(&$form, &$form_state) {
  if (db_result(db_query("SELECT 1 FROM {currencies} WHERE code = '%s'", $form_state['values']['code']))) {
    form_set_error('code', t('Error this currency code already exists.'));
  }
  if (!is_numeric($form_state['values']['decimal_places'])) {
    form_set_error('decimal_places', t('Decimal Place field can be numeric only.'));
  }
}

/**
 * Submission callback for the format_currency_add form.
 */
function format_currency_add_submit($form, &$form_state) {
  drupal_write_record('currencies', $form_state['values']);
  drupal_set_message(t('Successfully added currency %currency.', array('%currency' => $form_state['values']['code'])));
  $form_state['redirect'] = 'admin/settings/format_currency/overview';
}

/**
 * Form to edit a given currency in the database.
 */
function format_currency_edit(&$form_state, $currency) {
  $form['code'] = array(
    '#type' => 'textfield',
    '#title' => t('Currency code'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => $currency->code,
  );
  $form['symbol'] = array(
    '#type' => 'textfield',
    '#title' => t('Symbol'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => $currency->symbol,
  );
  $form['entity'] = array(
    '#type' => 'textfield',
    '#title' => t('HTML Entity'),
    '#size' => 10,
    '#maxlength' => 10,
    '#default_value' => $currency->entity,
  );
  $form['decimal_places'] = array(
    '#type' => 'textfield',
    '#title' => t('Decimal Places'),
    '#required' => TRUE,
    '#size' => 3,
    '#maxlength' => 2,
    '#default_value' => $currency->decimal_places,
  );
  $form['decimal_separator'] = array(
    '#type' => 'textfield',
    '#title' => t('Decimal Separator'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => $currency->decimal_separator,
  );
  $form['thousands_separator'] = array(
    '#type' => 'textfield',
    '#title' => t('Thousands Separator'),
    '#required' => TRUE,
    '#size' => 5,
    '#maxlength' => 5,
    '#default_value' => $currency->thousands_separator,
  );
  $form['symbol_position'] = array(
    '#type' => 'select',
    '#title' => t('Symbol Position'),
    '#options' => array(
      FORMAT_CURRENCY_LEFT => t('Left'),
      FORMAT_CURRENCY_RIGHT => t('Right'),
    ),
    '#required' => TRUE,
    '#default_value' => $currency->symbol_position,
  );
  $form['old_code'] = array(
    '#type' => 'hidden',
    '#value' => $currency->code,
  );

  return confirm_form($form, t('Edit Currency %currency:', array('%currency' => $currency->code)), 'admin/settings/format_currency/overview', ' ', t('Edit'));
}

/**
 * Validation callback for the format_currency_edit form.
 */
function format_currency_edit_validate(&$form, &$form_state) {
  if ($form_state['values']['old_code'] != $form_state['values']['code'] && db_result(db_query("SELECT 1 FROM {currencies} WHERE code = '%s'", $form_state['values']['code']))) {
    form_set_error('code', t('Error this currency code already exists.'));
  }
  if (!is_numeric($form_state['values']['decimal_places'])) {
    form_set_error('decimal_places', t('Decimal Place field can be numeric only.'));
  }
}

/**
 * Submission callback for the format_currency_edit form.
 */
function format_currency_edit_submit($form, &$form_state) {
  drupal_write_record('currencies', $form_state['values'], 'code');
  drupal_set_message(t('Successfully altered currency %currency.', array('%currency' => $form_state['values']['old_code'])));
  $form_state['redirect'] = 'admin/settings/format_currency/overview';
}

/**
 * Form to confirm deletion of a currency from the database.
 */
function format_currency_delete(&$form_state, $currency) {
  $form['code'] = array(
    '#type' => 'hidden',
    '#value' => $currency->code,
  );

  return confirm_form($form, t('Are you sure you want to delete currency %currency?', array('%currency' => $currency->code)), 'admin/settings/format_currency/overview');
}

/**
 * Submission callback for the format_currency_delete form.
 */
function format_currency_delete_submit($form, &$form_state) {
  db_query("DELETE FROM {currencies} WHERE code = '%s'", $form_state['values']['code']);
  drupal_set_message(t('Successfully deleted currency %currency.', array('%currency' => $form_state['values']['code'])));
  $form_state['redirect'] = 'admin/settings/format_currency/overview';
}

/**
 * Themes the admin overview form.
 */
function theme_format_currency_admin($form) {
  $output = '';
  $rows = array();
  $header = _format_currency_admin_header();

  foreach (element_children($form['currencies']) as $code) {
    $code = drupal_render($form['currencies'][$code]['code']);
    $position = drupal_render($form['currencies'][$code]['symbol_position']);
    $rows[] = array(
      check_plain($code),
      check_plain(drupal_render($form['currencies'][$code]['symbol'])),
      check_plain(drupal_render($form['currencies'][$code]['entity'])),
      check_plain(drupal_render($form['currencies'][$code]['decimal_places'])),
      check_plain(drupal_render($form['currencies'][$code]['decimal_separator'])),
      check_plain(drupal_render($form['currencies'][$code]['thousands_separator'])),
      _format_currency_symbol_position($position),
      l(t('edit'), "admin/settings/format_currency/$code/edit"),
      l(t('delete'), "admin/settings/format_currency/$code/delete"),
    );
  }

  if (empty($rows)) {
    $rows[][] = array('data' => t('No currencies are in the database.'), 'colspan' => count($header));
  }

  $output .= theme('table', $header, $rows);
  $output .= '<div>';
  $output .= l(t('Add new currency'), 'admin/settings/format_currency/overview/add');
  $output .= '</div>';
  $output .= theme('pager', array(), 25);

  // Render any remaining form elements.
  $output .= drupal_render($form);

  return $output;
}

/**
 * Helper form containing the admin overview table header.
 */
function _format_currency_admin_header() {
  return array(
    array('data' => t('Code'), 'field' => 'code', 'sort' => 'asc'),
    t('Symbol'),
    t('Entity'),
    t('Decimal Places'),
    t('Decimal Separator'),
    t('Thousands Separator'),
    t('Symbol Position'),
    array('data' => t('Operations'), 'colspan' => 2),
  );
}